/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * 
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is "SSH Console" code.
 * 
 * The Initial Developer of the Original Code is Todd Whiteman.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *   Todd Whiteman <twhitema@gmail.com> (original author)
 * 
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

// Global window variables.
var histList = [""];
var histPos = 0;
var _state = "disconnected"; // Either connected or disconnected.
var _ssh_channel = null; // xpcom python ssh channel
var _term_box_element = null;
var _term = null;
var question;
var _in;
var _out;

function onLoad() {
    try {
        _term_box_element = document.getElementById("term_box");
        _term = new VT100(80, 24, "term");
        _term.curs_set(true, true, _term_box_element);
        _term.noecho();

        // Replace the go_getch_ function with our own, this is called
        // for every keypress that is passed through the terminal to the
        // remote server. The character is already converted into the
        // required VT100 character sequence(s).
        VT100.go_getch_ = function() {
            var vt = VT100.the_vt_;
            if (vt === undefined) {
                    return;
            }
            var ch = vt.key_buf_.shift();
            //dump("go_getch_:: ch: '" + ch + "'\n");
            if (ch === undefined) {
                    return;
            }
            if (vt.echo_ && ch.length == 1) {
                    vt.addch(ch);
            }
            if (_ssh_channel) {
                _ssh_channel.sendStdin(ch);
            }
        }

        var serverTextbox = document.getElementById("sshconsole_server_textbox");
        // Get the "extensions.sshconsole." pref branch.
        var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                            getService(Components.interfaces.nsIPrefService);
        prefs = prefs.getBranch("extensions.sshconsole.");
        serverTextbox.value = prefs.getCharPref("lastConnection");
        serverTextbox.focus();

    } catch(ex) {
        alert("onLoad:: error: " + ex);
    }
}

function onUnload() {
    try {
        set_state("disconnected");
        // Get the "extensions.sshconsole." pref branch.
        var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                            getService(Components.interfaces.nsIPrefService);
        prefs = prefs.getBranch("extensions.sshconsole.");
        prefs.setCharPref("lastConnection",
                          document.getElementById("sshconsole_server_textbox").value);
    } catch(ex) {
        alert("onUnload:: error: " + ex);
    }
}

var sshChannelCallback = {
    onStartRequest: function(channel, ctxt) {
        //dump("onStartRequest\n");
    },
    onDataAvailable: function(channel, ctxt, input, sourceOffset, count) {
        try {
            /* Convert the input into unicode from UTF-8 encoding. */
            const replacementChar = Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER;
            var is = Components.classes["@mozilla.org/intl/converter-input-stream;1"]
                        .createInstance(Components.interfaces.nsIConverterInputStream);
            is.init(input, "UTF-8", 4096, replacementChar);
            var data = "";
            var str = {};
            while (is.readString(4096, str) != 0) {
                data += str.value;
            }
            /* Write the data to the terminal. */
            _term.write(data);
        } catch(ex) {
            dump("onDataAvailable exception: " + ex + "\n");
        }
    },
    onStopRequest: function(channel, ctxt, status) {
        //dump("onStopRequest\n");
    }
};

function set_state(new_state) {
    _state = new_state;
    var btn = document.getElementById("sshconsole_connect_button");
    if (new_state == "disconnected") {
        if (_ssh_channel) {
            btn.setAttribute("label", "Connect");
            _ssh_channel.cancel(Components.results.NS_OK);
            _ssh_channel = null;
        }
    } else {
        btn.setAttribute("label", "Disconnect");
        btn.blur();
        focusTerminal();
    }
}

function sshconsole_connect() {
    try {
        if (_state == "disconnected") {
            var uri_spec = document.getElementById("sshconsole_server_textbox").value;
            if (!uri_spec) {
                alert("You'll first need to enter the server name.");
                return;
            }
            if (uri_spec.substring(0, 7) != "sftp://") {
                uri_spec = "sftp://" + uri_spec + "/";
            }
            const ioSvc = Components.classes["@mozilla.org/network/io-service;1"].
                                getService(Components.interfaces.nsIIOService);
            var nsUri = ioSvc.newURI(uri_spec, null, null);
            var channel = ioSvc.newChannelFromURI(nsUri);
            _ssh_channel = channel.QueryInterface(Components.interfaces.pyISSHInteraction);
            _ssh_channel.asyncInteract(sshChannelCallback, "vt100", null);
    
            set_state("connected");
        } else {
    
            set_state("disconnected");
        }
    } catch(ex) {
        alert(ex);
    }
}

function focusTerminal() {
    _term_box_element.focus();
}
